/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include <zebra.h>
#include "vector.h"
#include "vty.h"
#include "command.h"
#include "getopt.h"
#include "thread.h"
#include "stream.h"
#include "network.h"
#include "prefix.h"
#include "version.h"
#include "memory.h"
#include "if.h"
#include "sockunion.h"
#include "log.h"
#include "pim.h"
#include "pimd.h"
#include "pim_fsm.h"
#include <netinet/ip_mroute.h>
#include "tables.h"
#include "pim_route.h"
#include <netinet/igmp.h>
#include "if_ioctl.h"
#include "pim_packet.h"
#include "igmp_def.h"
#include "if_idx_nexthop.h"

#include "zbug.h"

char b1[16], b2[16], b3[16], b4[16];  /* For inet_2a */
extern struct best_route best_route;

DEFUN (show_ip_rpf,
       show_ip_rpf_cmd,
       "show ip rpf A.B.C.D",
       "Show rpf route\n"
       "Add group permanent\n")
{
  struct interface *ifp;
  struct in_addr source;
  int ret;

  ret = inet_aton (argv[0], &source);
  if (!ret) {
      vty_out (vty, "Malformed address %s", VTY_NEWLINE);
      return CMD_WARNING;
    }
  
  /* 
     We look up in unicast route table now later on we can hopefully also use
     1) Static mroutes
     2) The mbgp table
 */

 if_idx_nexthop(&source);

 vty_out(vty, "RPF information for: %s%s", inet_ntoa(source), VTY_NEWLINE);

 if(best_route.idx < 0) {
   vty_out(vty, "  RPF fail -- no route found%s",  VTY_NEWLINE);
   return CMD_SUCCESS;
 }
 else {
   
   if((ifp = get_iflist_ifp(best_route.idx))) {
     vty_out(vty, "  RPF interface: %s%s", ifp->name, VTY_NEWLINE);
   }
   else {
     vty_out(vty, "  RPF interface index: %d WARNING PIM not enabled%s",
	     best_route.idx, VTY_NEWLINE);
   }
   vty_out(vty, "  RPF neighbor: %s", inet_2a(best_route.gw, b1));
   if(best_route.gw == 0) vty_out(vty, " - directly connected");
   
   vty_out(vty, "%s",VTY_NEWLINE);
   vty_out(vty, "  RPF route/mask: %s/%d%s", inet_2a(best_route.dst, b1), 
	   best_route.mlen, VTY_NEWLINE);
   vty_out(vty, "  RPF type: %s%s", "unicast", VTY_NEWLINE);
 }
 return CMD_SUCCESS;
}

void
rpf_init()
{
 install_element (VIEW_NODE, &show_ip_rpf_cmd); 
 install_element (ENABLE_NODE, &show_ip_rpf_cmd); 
}
